@@ -72,21 +72,28 @@ def arggetter(func, _cache={}):
     if func in _cache:
         return _cache[func]
 
-    argnames = get_argnames(func)
-    argcount = len(argnames)
+    original = getattr(func, '__original__', None) or unwrap(func)
+    argnames = get_argnames(original)
+    indexes = dict((name, i) for i, name in enumerate(argnames))
+    defaults_tuple = original.__defaults__
+    if defaults_tuple:
+        defaults = dict(zip(argnames[-len(defaults_tuple):], defaults_tuple))
+    else:
+        defaults = {}
 
     def get_arg(name, args, kwargs):
-        if name not in argnames:
+        if name not in indexes:
             raise TypeError("%s() doesn't have argument named %s" % (func.__name__, name))
         else:
-            if name in kwargs:
+            index = indexes[name]
+            if index < len(args):
+                return args[index]
+            elif name in kwargs:
                 return kwargs[name]
-            elif name in argnames:
-                index = argnames.index(name)
-                if index < len(args):
-                    return args[index]
-                else:
-                    return func.__defaults__[index - argcount]
+            elif name in defaults:
+                return defaults[name]
+            else:
+                raise TypeError("%s() missing required argument: '%s'" % (func.__name__, name))
 
     _cache[func] = get_arg
     return get_arg
